@@ -12,7 +12,7 @@
    cat("\n")
    cat("### Mixture of methods including mixtures\n")
    cat("### Arnost Komarek\n\n")
-   cat("### See citation(\"mixAK\") for the best way to cite\n")
+   cat("### See citation(\"mixAK\") or toBibtex(citation(\"mixAK\")) for the best way to cite\n")
    cat("### the package if you find it useful.\n\n")
    invisible()
 }
